#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "posix-queue.hxx"
#include "stopwatch.h"



//
// experimental: Try the use select(2) system call to wait 
//               for a chunk of messages
//
#if 0
#define USE_SELECT
#define CHUNK_SIZE   64
#endif



int main(int argc, char **argv)
{
  Queue *pQueue = 0;
  FILE *pOutfile = 0;
  char aBuffer[512];
  size_t Length;
  DEFINE_STOPWATCH(Rolex)
  double dTimeInSeconds;
  unsigned long ulMessagesReceived = 0;
  unsigned long ulBytesReceived = 0;
  unsigned long ulMessagesWrittenToFile = 0;

  //
  // This is the recommended locking scheme to ensure that at most one
  // eater is running at a time.
  // Note that the lock will vanish on process exit, automatically.
  // Modern POSIX semaphores lack this feature. System V semaphores 
  // support SEM_UNDO, but this interface is rather old. :-(
  //
  int l = creat("./lockfile",0666);
  if (lockf(l,F_TLOCK,0))
  {
    perror("lockf");
    fprintf(stderr,"eater: Other eater is holding the lock, sorry.\n");
    return 0;
  }

  pQueue = new PosixQueue(std::string("TestQueue"));
  if (pQueue->Open())
  {
    fprintf(stderr,"eater: could not open message queue\n");
    return -1;
  }
  printf("eater: Current message count is %ld.\n",pQueue->GetMessageCount());

  if (1 < argc)
  {
    if ((pOutfile = fopen(argv[1],"w")) == 0)
    {
      fprintf(stderr,"eater: could not open output file \"%s\"\n",argv[1]);
    }
  }

#ifdef USE_SELECT

  const unsigned long ulChunkSize = CHUNK_SIZE;
  unsigned long ulMessagesToBeEaten = 0;
  unsigned long ulSelectCalls = 0;

  pQueue->SetNonblockingMode();

#endif

  printf("getting messages...\n");
  for (;;)
  {

#ifdef USE_SELECT
 
    if (ulMessagesToBeEaten == 0)
    {
      //
      // just to try it: use select system call to wait for incoming messages
      //
      // This feature could be useful if there are other message sources and
      // multiplexing is required.
      // Note that this imposes some overhead on getting messages, especially
      // if select is called before each PosixQueue::Get. 
      // A chunk of messages will tried to be read on the return of select.
      //
 
      int fd = dynamic_cast<PosixQueue *>(pQueue)->GetFileDescriptor();
      fd_set readfds;
      fd_set writefds;
      fd_set exceptfds;
 
      FD_ZERO(&readfds);
      FD_ZERO(&writefds);
      FD_ZERO(&exceptfds);
      FD_SET(fd,&readfds);
      ulMessagesToBeEaten = ulChunkSize;
      ulSelectCalls++;
      int k = select(fd + 1,&readfds,&writefds,&exceptfds,0);
      if (k == -1)
      {
        perror("select");
        break;
      }
    }

#endif  

    if (!pQueue->Get(aBuffer,512,Length))
    {
      if (!strncmp(aBuffer,"Start",6))
      {
        ulMessagesReceived = 0;
        ulBytesReceived = 0;
        START_STOPWATCH(Rolex)
      }
      else if (!strncmp(aBuffer,"Check",6))
      {
        STOP_STOPWATCH(Rolex)
        LOOK_AT_STOPWATCH(Rolex,dTimeInSeconds)
        printf("eater: %lu messages (total of %lu MB) got within %.1lfs (%.0lf Msg/s)\n",
               ulMessagesReceived,
               ulBytesReceived / 1000000,
               dTimeInSeconds,
               ulMessagesReceived / dTimeInSeconds);
        if (1 < argc)
	{
	  printf("eater: %lu messages written to %s\n",ulMessagesWrittenToFile,argv[1]);
        }
      }
      else if (!strncmp(aBuffer,"exit",4) || !strncmp(aBuffer,"quit",4))
      {
        printf("eater: exiting on feeder request...\n");
        break;
      }
      else if (aBuffer[0])
      {
        puts(aBuffer);
      }
      else
      {
        if (pOutfile != 0)
	{
          if (fwrite(aBuffer,Length,1,pOutfile) == 1)
	  {
            ulMessagesWrittenToFile++;
          }
        }
        ulMessagesReceived++;
        ulBytesReceived += Length;
      }  

#ifdef USE_SELECT

      if (ulMessagesToBeEaten)
      {
        ulMessagesToBeEaten--;
      }

#endif

    }
    else
    {

#ifdef USE_SELECT

      //
      // Get failed - nothing more to eat for now
      //
      ulMessagesToBeEaten = 0;

#else

      printf("eater: PosixQueue::Get failed\n");
      break;

#endif

    }
  }

  if (pOutfile)
  {
    fclose(pOutfile);
  }

  //
  // delete queue object - but not the queue itself,
  // mq_unlink would do this
  //
  delete pQueue;

  //
  // unlock - but is unecessary because the system automatically
  // removes the held lock on program exit
  //
  lockf(l,F_ULOCK,0);

#ifdef USE_SELECT

  printf("eater: number of select system calls: %lu\n",ulSelectCalls);

#endif

  return 0;
}
