#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "queue.hxx"
#include "posix-queue.hxx"
#include "stopwatch.h"



int main(int argc, char **argv)
{
  Queue *pQueue = 0;
  char aBuffer[512];
  DEFINE_STOPWATCH(Rolex)
  double TimeInSeconds;
  unsigned long ulMessagesToBeSent = 1;
  unsigned long ulMessagesSent = 0;
  unsigned long ulMessagesNotSent = 0;
  
  pQueue = new PosixQueue(std::string("TestQueue"));
  if (pQueue->Open())
  {
    fprintf(stderr,"feeder: could not open message queue\n");
    return -1;
  }

  if (argc == 1)
  {
    //
    // send a message burst, request time measurement at eater side
    //
    ulMessagesToBeSent = 10000000;     // ten millions
    memset(aBuffer,0,sizeof(aBuffer));
    printf("putting messages...\n");
    if (!pQueue->Put("Start",6))
    {
      //
      // send messages in nonblocking mode
      //
      if (pQueue->SetNonblockingMode())
      {
        fprintf(stderr,"feeder: could not set nonblocking mode\n");
      }
      START_STOPWATCH(Rolex);
      for (unsigned long i = 0; i < ulMessagesToBeSent; i++)
      {
        //
        // insist in sending the message, retry as many times as needed
        //
        while (pQueue->Put(aBuffer,128))
        {
	  ulMessagesNotSent++;
        }
	ulMessagesSent++;
      }

      //
      // enter blocking mode to make sure the "Check" goes through,
      // print time and counters
      //
      pQueue->SetBlockingMode();
      pQueue->Put("Check",6);
      STOP_STOPWATCH(Rolex)
      LOOK_AT_STOPWATCH(Rolex,TimeInSeconds)
      printf("%ld messages put within %.1lfs\n",ulMessagesSent,TimeInSeconds);
      if (ulMessagesNotSent)
      {
        printf("%lu put retrie%s needed (%.0lf%%)\n",
               ulMessagesNotSent, (1 < ulMessagesNotSent) ? "s" : "",
               100.0 * ulMessagesNotSent / ulMessagesSent);
      }
    }
    else
    {
      printf("...Sorry, unable to put messages at all.\n");
    }
  }
  else
  {
    //
    // send argv[1], argv{2], ... as text messages
    //
    for (int i = 1; i < argc; i++)
    {
      if (!pQueue->Put(argv[i],strlen(argv[i]) + 1))
      {
	ulMessagesSent++;
      }
      else
      {
  	ulMessagesNotSent++;
      }
    }
  }

  delete pQueue;

  return 0;
}
