/*      
**  stopwatch.h 
**    
**  Facility:  
**        
**      auxiliary stopwatch macro package
**
**  Authors:
**
**      ASI     Armin Singer, EQUIcon Software GmbH
**      
**  Module Description:
**
**      Not a modul, just a macro package.
**      Sometimes keeping track of time is required.
**      The stopwatch package supports it.
**
**  Interfaces:
**
**      There are macros to define, start, stop, and look at a stopwatch
**      "object". The header time.h is required to be included before
**      the stopwatch package.
** 
**  Creation Date:  
**  
**      25-OCT-2000
**
**  Modification History:
**
**      DD-MMM-YYYY WHO
**
**          what has been modified
**
**
*/

#ifndef ____STOPWATCH_H__INCLUDED____
#define ____STOPWATCH_H__INCLUDED____



struct stopwatch_descriptor 
{
    struct timeval start_time;
    struct timeval stop_time;
    struct timeval elapsed_time;
};



#define DEFINE_STOPWATCH(stopwatch)                                        \
    struct stopwatch_descriptor stopwatch = {{0,0},{0,0},{0,0}};

#define START_STOPWATCH(stopwatch)                                         \
    {                                                                      \
        stopwatch.elapsed_time.tv_sec  = 0;                                \
        stopwatch.elapsed_time.tv_usec = 0;                                \
        gettimeofday(&stopwatch.start_time,NULL);                          \
        stopwatch.stop_time = stopwatch.start_time;                        \
    }

#define STOP_STOPWATCH(stopwatch)                                          \
    {                                                                      \
        gettimeofday(&stopwatch.stop_time,NULL);                           \
    }  

#define LOOK_AT_STOPWATCH(stopwatch,seconds)                               \
    {                                                                      \
        stopwatch.elapsed_time.tv_sec      = stopwatch.stop_time.tv_sec -  \
                                             stopwatch.start_time.tv_sec;  \
        if (stopwatch.start_time.tv_usec  <= stopwatch.stop_time.tv_usec)  \
        {                                                                  \
            stopwatch.elapsed_time.tv_usec = stopwatch.stop_time.tv_usec - \
                                             stopwatch.start_time.tv_usec; \
        }                                                                  \
        else                                                               \
        {                                                                  \
            stopwatch.elapsed_time.tv_usec = stopwatch.stop_time.tv_usec + \
                                             1000000 -                     \
                                             stopwatch.start_time.tv_usec; \
            stopwatch.elapsed_time.tv_sec--;                               \
        }                                                                  \
        seconds = (double)stopwatch.elapsed_time.tv_sec +                  \
                  1.0e-6 * stopwatch.elapsed_time.tv_usec;                 \
    }



#endif  /* ____STOPWATCH_H__INCLUDED____ */
